// Copyright (2013) Jann Horn <jann@thejh.net>

#include <stdio.h>
#include <stdlib.h>
#include <assert.h>
#include <string.h>
#include <errno.h>

HEADER typedef struct {
HEADER   int width, height;
HEADER   unsigned char *data;
HEADER   int data_size;
HEADER } pnm_image;

PUBLIC_FN pnm_image *read_image(char *path) {
  int saved_errno = 0;
  
  // prepare stuff
  FILE *f = fopen(path, "r");
  if (!f) return NULL;
  pnm_image *result = malloc(sizeof(pnm_image));
  if (!result) { saved_errno = errno; goto out_close; }
  
  // read header
  int line = 0;
  int bufsize = 1024;
  char *buf = malloc(bufsize);
  while (line < 3) {
    assert(fgets(buf, bufsize, f) == buf);
    if (buf[0] == '#') continue;
    switch (line) {
      case 0: {
        if (strcmp(buf, "P6\n")) {
          saved_errno = ENOEXEC;
          free(result);
          goto out_freebuf;
        }
        break;
      }
      case 1: {
        sscanf(buf, "%i %i", &result->width, &result->height);
        result->data_size = 3 * result->width * result->height;
        result->data = malloc(result->data_size);
        // FIXME NULL check?
        break;
      }
      case 2: {
        if (strcmp(buf, "255\n")) {
          saved_errno = EINVAL;
          free(result->data);
          free(result);
          goto out_freebuf;
        }
        break;
      }
      default: assert(0);
    }
    line++;
  }
  
  // read pixel data
  assert(fread(result->data, result->data_size, 1, f) == 1);

  // clean up
out_freebuf:
  free(buf);
out_close:
  fclose(f);
  if (saved_errno) errno = saved_errno;
  return result;
}

PUBLIC_FN int write_image(pnm_image *img, char *outfile) {
  char outbuf[100+img->data_size];
  int ob_used = snprintf(outbuf, sizeof(outbuf), "P6\n%d %d\n255\n", img->width, img->height);
  if (ob_used >= 100) xperror("header size is bigger than anticipated", 0);
  memcpy(outbuf+ob_used, img->data, img->data_size);
  return write_file(outfile, outbuf, ob_used+img->data_size, 0);
}

PUBLIC_FN pnm_image *dup_image(pnm_image *img) {
  pnm_image *r = malloc(sizeof(pnm_image));
  if (!r) return NULL;
  *r = *img;
  r->data = malloc(r->data_size);
  if (!r->data) { free(r); return NULL; }
  memcpy(r->data, img->data, r->data_size);
  return r;
}
